from .dgutLogin import dgutLogin
import requests
import re
import lxml.etree
import json
import datetime


# 登录装饰器
def xgxtt_decorator_signin(func):
    '''
    定义一个登录学工系统的装饰器
    '''

    def wrapper(self, *args, **kargs):
        response = self.signin('xgxtt')
        # 情况处理
        if response['code'] == 1:
            return func(self, *args, **kargs)
        else:
            return response
    return wrapper


class dgutXgxtt(dgutLogin):
    '''
    学工系统类

    username : 中央认证账号用户名
    password : 中央认证账号密码
    '''

    def __init__(self, username, password):
        dgutLogin.__init__(self, username, password)
        self.xgxtt = {
            'headers': {
                'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/87.0.4280.141 Safari/537.36',
                'Host': 'stu.dgut.edu.cn',
            },
            # 系统主页
            'homepage': {
                'url': 'http://stu.dgut.edu.cn/homepage.jsp',
                'response': [],
            },
            # 考勤界面
            'attendance_main': {
                'url': 'http://stu.dgut.edu.cn/student/partwork/attendance_main.jsp',
                'response': [],
            },
            # 考勤请求
            'attendance': {
                'url': 'http://stu.dgut.edu.cn/student/partwork/attendance.jsp',
                'response': [],
            },
        }

    def __str__(self):
        addr = hex(id(self))
        return f"<dgutXgxtt at 0x{addr[2:].upper().zfill(16)} username is {self.username}>"

    @xgxtt_decorator_signin
    def get_workAssignment(self):
        '''
        获取考勤职位信息
        '''
        try:
            # 请求考勤页职位信息
            response = self.session.get(
                self.xgxtt['attendance']['url'], headers=self.xgxtt['headers'], timeout=self.timeout)
            self.xgxtt['attendance']['response'].append([response])

            if response.status_code != 200:
                return {'message': '页面请求失败', 'code': 300}

            html = lxml.etree.HTML(response.text)
            result = html.xpath('//div[@class="searchTitle"]')
            works = result[0].xpath('./select/option[position()>1]')
            workAssignment = []
            if not len(works):
                return {'message': '请求的职位信息为空', 'code': 304}
            for item in works:
                id_ = item.xpath('./@value')
                name = item.xpath('./text()')
                workAssignment.append((id_[0], name[0]))

            return {'message': '获取职位信息成功', 'code': 1, 'workAssignment': workAssignment}

        except requests.exceptions.ConnectTimeout:
            # 请求超时
            return {'message': '请求超时', 'code': 303}

    @xgxtt_decorator_signin
    def attendance(self, flag: int, workAssignmentId: int = None):
        '''
        学工系统考勤

        attendance(self, flag[, workAssignmentId])
        flag : 0->模拟到选择职位那一步，用于测试 / 1->签到 / 2->签退
        workAssignmentId : 职位id，为None时自动获取当前的首个职位
        '''
        try:
            self.xgxtt['attendance']['response'].append(list())

            if not flag in [0, 1, 2]:
                return {'message': '参数错误：flag只能是0、1或2', 'code': 2}
            if flag == 0:
                action_name = ''
                s = '测试'
            if flag == 1:
                action_name = 'beginWork'
                s = '签到'
            if flag == 2:
                action_name = 'endWork'
                s = '签退'

            # 如果没有传递workAssignmentId，自动获取可考勤的第一个职位作为考勤职位
            if not workAssignmentId:
                workAssignment = self.get_workAssignment()
                if workAssignment['code'] != 1:
                    message = workAssignment['message']
                    return {'message': f'请求信息失败：{message}', 'code': 305}
                workAssignmentId = workAssignment['workAssignment'][0][0]

            # 请求考勤界面
            response = self.session.get(
                self.xgxtt['attendance']['url'], headers=self.xgxtt['headers'], timeout=self.timeout)
            self.xgxtt['attendance']['response'][-1].append(response)
            if response.status_code != 200:
                return {'message': '考勤页面请求失败', 'code': 300}

            # 获取session_token
            session_token = re.search(
                r'session_token.*?value="(.*?)"/>', response.text, re.S).group(1)

            # 提交表单数据
            data = {
                'action_name': action_name,
                'modifying': 'true',
                'salaryInfoId': '',
                'session_token': session_token,
                'backUrl': '',
                'workAssignmentId': workAssignmentId
            }

            # 发送请求
            response = self.session.post(
                self.xgxtt['attendance']['url'], headers=self.xgxtt['headers'], data=data, timeout=self.timeout)
            self.xgxtt['attendance']['response'][-1].append(response)

            if response.status_code != 200:
                return {'message': f'考勤{s}请求失败', 'code': 300}
            sign_time = datetime.datetime.now()
            return {
                'message': f'{s}成功',
                'code': 1,
                'info': {
                    'data': data,
                    'time': sign_time,
                }
            }

        except AttributeError:
            # 属性错误
            return {'message': '页面session_token请求失败', 'code': 301}
        except requests.exceptions.ConnectTimeout:
            # 请求超时
            return {'message': '请求超时', 'code': 303}



import requests
import re
import json


class dgutLogin(object):
    '''
    登录类
    username : 中央认证账号用户名
    password : 中央认证账号密码
    '''

    def __init__(self, username: str, password: str):
        '''
        构造函数

        __init__(self, username, password)
        username : 中央认证账号用户名
        password : 中央认证账号密码
        '''
        self.username = str(username)
        self.password = str(password)

        # 创建一个会话
        self.session = requests.Session()
        # 设置请求超时时间
        self.timeout = 20
        # 目前收录的系统名
        '''
        学工系统：xgxtt
        教务网络管理系统：jwyd
        疫情防控系统：illness
        事务中心：stuaffair
        '''
        self.sys = ['xgxtt', 'jwyd', 'illness', 'stuaffair']
        '''
        login属性，对应登录的各项信息
        headers是http requests的headers
        各系统名对应的字典分别是登录url和存放对象请求记录的response
        '''
        self.login = {
            'headers': {
                'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/87.0.4280.141 Safari/537.36',
            },
            'xgxtt': {
                'url': r'https://cas.dgut.edu.cn/home/Oauth/getToken/appid/xgxtt.html',
                'response': [],
            },
            'jwyd': {
                'url': r'https://cas.dgut.edu.cn/home/Oauth/getToken/appid/jwyd.html',
                'response': [],
            },
            'illness': {
                'url': r'https://cas.dgut.edu.cn/home/Oauth/getToken/appid/illnessProtectionHome/state/home.html',
                'response': [],
            },
            'stuaffair': {
                'url': r'https://cas.dgut.edu.cn/home/Oauth/getToken/appid/stuaffair.html',
                'response': [],
            },
        }

    def __str__(self):
        addr = hex(id(self))
        return f"<dgutLogin at 0x{addr[2:].upper().zfill(16)} username is {self.username}>"

    def signin(self, sys_name: str = None):
        '''
        登录函数

        signin(self[, sys_name])
        sys_name : 系统名 | None=="xgxtt"
        '''
        if not sys_name:
            url = self.login['xgxtt']['url']
            sys_name = 'xgxtt'
        if not sys_name in self.sys:
            return {'message': '参数错误：系统名', 'code': 2}
        url = self.login[sys_name]['url']
        self.login[sys_name]['response'].append(list())
        try:
            # 获取登录token
            response = self.session.get(
                url, headers=self.login['headers'], timeout=self.timeout)
            self.login[sys_name]['response'][-1].append(response)
            if response.status_code != 200:
                return {'message': f'登录页面{url}请求失败', 'code': 300}

            if re.search('token = \".*?\"', response.text):
                # 如果获取到登录token，说明还未登录
                __token__ = re.search(
                    'token = \"(.*?)\"', response.text).group(1)
                # 发送登录请求
                data = {
                    'username': self.username,
                    'password': self.password,
                    '__token__': __token__
                }
                response = self.session.post(
                    url, data=data, headers=self.login['headers'], timeout=self.timeout)
                self.login[sys_name]['response'][-1].append(response)
                result = json.loads(response.text)

                if result['code'] == 1:
                    # 登录成功
                    # 认证
                    auth_url = result['info']
                    response = self.session.get(
                        auth_url, headers=self.login['headers'])
                    self.login[sys_name]['response'][-1].append(response)
                    if response.status_code == 200:
                        # 认证成功
                        return result
                    else:
                        return {'message': f'认证{auth_url}失败', 'code': 302}
                else:
                    if result['code'] == 8:
                        return {'message': '密码错误', 'code': 400}
                    elif result['code'] == 15:
                        return {'message': '不存在该用户', 'code': 401}
                    else:
                        return {'message': '未知的登录错误', 'code': 402, 'info': json.loads(response.text)}

            else:
                # 已登陆过，只需要获取认证
                info = None
                if re.match(r'.*?token.*?', response.url, re.S):
                    info = response.url.strip()
                if info:
                    return {'message': '验证通过', 'code': 1, 'info': info}
                for item in response.history:
                    if re.match(r'.*?token.*?', item.url, re.S):
                        info = item.url.strip()
                if not info:
                    return {'message': f'认证{url}失败', 'code': 302}
                return {'message': '验证通过', 'code': 1, 'info': info}

        except AttributeError:
            # 属性错误
            return {'message': '页面token请求失败', 'code': 301}
        except requests.exceptions.ConnectTimeout:
            # 请求超时
            return {'message': '请求超时', 'code': 303}

    def add_sys(self, sys_name: str, login_url: str):
        '''
        添加系统

        sys_name : 系统名
        login_url: 系统登录url
        '''
        if not re.match("^https?://.*?", str(login_url)):
            return None
        self.sys.append(str(sys_name))
        self.login[str(sys_name)] = {
            'url': str(login_url),
            'response': [],
        }
        return self.login[str(sys_name)]
